@@ -3,6 +3,9 @@
 
 namespace Icinga\Module\Setup;
 
+use Icinga\Application\Platform;
+use Icinga\Module\Setup\Requirement\SetRequirement;
+use Icinga\Module\Setup\Requirement\WebLibraryRequirement;
 use PDOException;
 use Icinga\Web\Form;
 use Icinga\Web\Wizard;
@@ -31,8 +34,6 @@ use Icinga\Module\Setup\Steps\UserGroupStep;
 use Icinga\Module\Setup\Utils\EnableModuleStep;
 use Icinga\Module\Setup\Utils\DbTool;
 use Icinga\Module\Setup\Requirement\OSRequirement;
-use Icinga\Module\Setup\Requirement\ClassRequirement;
-use Icinga\Module\Setup\Requirement\PhpConfigRequirement;
 use Icinga\Module\Setup\Requirement\PhpModuleRequirement;
 use Icinga\Module\Setup\Requirement\PhpVersionRequirement;
 use Icinga\Module\Setup\Requirement\ConfigDirectoryRequirement;
@@ -89,7 +90,8 @@ class WebWizard extends Wizard implements SetupWizard
         'icingaweb_group',
         'icingaweb_group_membership',
         'icingaweb_user',
-        'icingaweb_user_preference'
+        'icingaweb_user_preference',
+        'icingaweb_rememberme'
     );
 
     /**
@@ -227,28 +229,13 @@ class WebWizard extends Wizard implements SetupWizard
                     unset($pageData['setup_usergroup_backend']);
                 }
             }
-        } elseif ($page->getName() === 'setup_general_config') {
-            $authData = $this->getPageData('setup_authentication_type');
-            if ($authData['type'] === 'db') {
-                $page
-                    ->create($this->getRequestData($page, $request))
-                    ->getElement('global_config_backend')
-                    ->setValue('db');
-                $page->info(
-                    mt(
-                        'setup',
-                        'Note that choosing "Database" as preference storage causes'
-                        . ' Icinga Web 2 to use the same database as for authentication.'
-                    ),
-                    false
-                );
-            }
-        } elseif ($page->getName() === 'setup_authentication_type' && $this->getDirection() === static::FORWARD) {
+        } elseif ($page->getName() === 'setup_authentication_type') {
             $authData = $this->getPageData($page->getName());
+            $pageData = & $this->getPageData();
+
             if ($authData !== null && $request->getPost('type') !== $authData['type']) {
                 // Drop any existing page data in case the authentication type has changed,
                 // otherwise it will conflict with other forms that depend on this one
-                $pageData = & $this->getPageData();
                 unset($pageData['setup_admin_account']);
                 unset($pageData['setup_authentication_backend']);
 
@@ -259,6 +246,13 @@ class WebWizard extends Wizard implements SetupWizard
                     unset($pageData['setup_config_db_resource']);
                     unset($pageData['setup_config_db_creation']);
                 }
+            } elseif (isset($authData['type']) && $authData['type'] == 'external') {
+                // If you choose the authentication type external and validate the database and then come
+                // back to change the authentication type but do not change it, you will get an database configuration
+                // related error message on the next page. To avoid this error, the 'setup_config_db_resource'
+                // page must be unset.
+
+                unset($pageData['setup_config_db_resource']);
             }
         }
     }
@@ -295,18 +289,20 @@ class WebWizard extends Wizard implements SetupWizard
             $skip = $backendConfig['backend'] !== 'ldap';
         } elseif ($newPage->getName() === 'setup_config_db_resource') {
             $authData = $this->getPageData('setup_authentication_type');
-            $configData = $this->getPageData('setup_general_config');
-            $skip = $authData['type'] === 'db' || $configData['global_config_backend'] !== 'db';
+            $skip = $authData['type'] === 'db';
         } elseif (in_array($newPage->getName(), array('setup_auth_db_creation', 'setup_config_db_creation'))) {
             if (($newPage->getName() === 'setup_auth_db_creation' || $this->hasPageData('setup_config_db_resource'))
                 && (($config = $this->getPageData('setup_auth_db_resource')) !== null
                     || ($config = $this->getPageData('setup_config_db_resource')) !== null)
-                    && !$config['skip_validation']
+                    && !$config['skip_validation'] &&  $this->getDirection() == static::FORWARD
             ) {
+                // Execute this code only if the direction is forward.
+                // Otherwise, an error will be output when you go back.
                 $db = new DbTool($config);
 
                 try {
                     $db->connectToDb(); // Are we able to login on the database?
+
                     if (array_search(reset($this->databaseTables), $db->listTables(), true) === false) {
                         // In case the database schema does not yet exist the
                         // user needs the privileges to setup the database
@@ -367,7 +363,10 @@ class WebWizard extends Wizard implements SetupWizard
             'setup_auth_db_resource',
             'setup_config_db_resource',
             'setup_ldap_resource',
-            'setup_monitoring_ido', // TODO(mf): This should be handled by MonitoringWizard
+            'setup_monitoring_ido',
+            'setup_icingadb_resource',
+            'setup_icingadb_redis',
+            'setup_icingadb_api_transport'
         ))) {
             $page->addElement(
                 'submit',
@@ -376,13 +375,13 @@ class WebWizard extends Wizard implements SetupWizard
                     'ignore'                => true,
                     'label'                 => t('Validate Configuration'),
                     'data-progress-label'   => t('Validation In Progress'),
-                    'decorators'            => array('ViewHelper')
+                    'decorators'            => array('ViewHelper'),
+                    'formnovalidate'        => 'formnovalidate'
                 )
             );
             $page->getDisplayGroup('buttons')->addElement($page->getElement('backend_validation'));
         }
 
-        // TODO(mf): This should be handled by MonitoringWizard
         if ($page->getName() === 'setup_command_transport') {
             if ($page->getSubForm('transport_form')->getSubForm('transport_form') instanceof ApiTransportForm) {
                 $page->addElement(
@@ -392,7 +391,8 @@ class WebWizard extends Wizard implements SetupWizard
                         'ignore'                => true,
                         'label'                 => t('Validate Configuration'),
                         'data-progress-label'   => t('Validation In Progress'),
-                        'decorators'            => array('ViewHelper')
+                        'decorators'            => array('ViewHelper'),
+                        'formnovalidate'        => 'formnovalidate'
                     )
                 );
                 $page->getDisplayGroup('buttons')->addElement($page->getElement('transport_validation'));
@@ -568,11 +568,11 @@ class WebWizard extends Wizard implements SetupWizard
         $set = new RequirementSet();
 
         $set->add(new PhpVersionRequirement(array(
-            'condition'     => array('>=', '5.6'),
-            'description'   => mt(
+            'condition'     => array('>=', '7.2'),
+            'description'   => sprintf(mt(
                 'setup',
-                'Running Icinga Web 2 requires PHP version 5.6.'
-            )
+                'Running Icinga Web 2 requires PHP version %s.'
+            ), '7.2')
         )));
 
         $set->add(new OSRequirement(array(
@@ -585,6 +585,24 @@ class WebWizard extends Wizard implements SetupWizard
             )
         )));
 
+        $set->add(new WebLibraryRequirement(array(
+            'condition'     => ['icinga-php-library', '>=', '0.9.0'],
+            'alias'         => 'Icinga PHP library',
+            'description'   => mt(
+                'setup',
+                'The Icinga PHP library (IPL) is required for Icinga Web 2 and modules'
+            )
+        )));
+
+        $set->add(new WebLibraryRequirement(array(
+            'condition'     => ['icinga-php-thirdparty', '>=', '0.11.0'],
+            'alias'         => 'Icinga PHP Thirdparty',
+            'description'   => mt(
+                'setup',
+                'The Icinga PHP Thirdparty library is required for Icinga Web 2 and modules'
+            )
+        )));
+
         $set->add(new PhpModuleRequirement(array(
             'condition'     => 'OpenSSL',
             'description'   => mt(
@@ -594,7 +612,14 @@ class WebWizard extends Wizard implements SetupWizard
         )));
 
         $set->add(new PhpModuleRequirement(array(
-            'optional'      => true,
+            'condition'     => 'XML',
+            'description'   => mt(
+                'setup',
+                'The XML module for PHP is required for Markdown and custom HTML annotations.'
+            )
+        )));
+
+        $set->add(new PhpModuleRequirement(array(
             'condition'     => 'JSON',
             'description'   => mt(
                 'setup',
@@ -602,6 +627,30 @@ class WebWizard extends Wizard implements SetupWizard
             )
         )));
 
+        $set->add(new PhpModuleRequirement(array(
+            'condition'     => 'gettext',
+            'description'   => mt(
+                'setup',
+                'For message localization, the gettext module for PHP is required.'
+            )
+        )));
+
+        $set->add(new PhpModuleRequirement(array(
+            'condition'     => 'INTL',
+            'description'   => mt(
+                'setup',
+                'For language, timezone and date/time format negotiation, the INTL module for PHP is required.'
+            )
+        )));
+
+        $set->add(new PhpModuleRequirement(array(
+            'condition'     => 'DOM',
+            'description'   => mt(
+                'setup',
+                'For charts and exports of views and reports to PDF, the DOM module for PHP is required.'
+            )
+        )));
+
         $set->add(new PhpModuleRequirement(array(
             'optional'      => true,
             'condition'     => 'LDAP',
@@ -613,21 +662,10 @@ class WebWizard extends Wizard implements SetupWizard
 
         $set->add(new PhpModuleRequirement(array(
             'optional'      => true,
-            'condition'     => 'INTL',
+            'condition'     => 'mbstring',
             'description'   => mt(
                 'setup',
-                'If you want your users to benefit from language, timezone and date/time'
-                . ' format negotiation, the INTL module for PHP is required.'
-            )
-        )));
-
-        // TODO(6172): Remove this requirement once we do not ship dompdf with Icinga Web 2 anymore
-        $set->add(new PhpModuleRequirement(array(
-            'optional'      => true,
-            'condition'     => 'DOM',
-            'description'   => mt(
-                'setup',
-                'To be able to export views and reports to PDF, the DOM module for PHP is required.'
+                'In case you want views being exported to PDF, you\'ll need the mbstring extension for PHP.'
             )
         )));
 
@@ -649,8 +687,8 @@ class WebWizard extends Wizard implements SetupWizard
             )
         )));
 
-        $mysqlSet = new RequirementSet(true);
-        $mysqlSet->add(new PhpModuleRequirement(array(
+        $dbSet = new RequirementSet(false, RequirementSet::MODE_OR);
+        $dbSet->add(new PhpModuleRequirement(array(
             'optional'      => true,
             'condition'     => 'pdo_mysql',
             'alias'         => 'PDO-MySQL',
@@ -659,29 +697,7 @@ class WebWizard extends Wizard implements SetupWizard
                 'To store users or preferences in a MySQL database the PDO-MySQL module for PHP is required.'
             )
         )));
-        $mysqlSet->add(new ClassRequirement(array(
-            'optional'      => true,
-            'condition'     => 'Zend_Db_Adapter_Pdo_Mysql',
-            'alias'         => mt('setup', 'Zend database adapter for MySQL'),
-            'description'   => mt(
-                'setup',
-                'The Zend database adapter for MySQL is required to access a MySQL database.'
-            ),
-            'textAvailable' => mt(
-                'setup',
-                'The Zend database adapter for MySQL is available.',
-                'setup.requirement.class'
-            ),
-            'textMissing'   => mt(
-                'setup',
-                'The Zend database adapter for MySQL is missing.',
-                'setup.requirement.class'
-            )
-        )));
-        $set->merge($mysqlSet);
-
-        $pgsqlSet = new RequirementSet(true);
-        $pgsqlSet->add(new PhpModuleRequirement(array(
+        $dbSet->add(new PhpModuleRequirement(array(
             'optional'      => true,
             'condition'     => 'pdo_pgsql',
             'alias'         => 'PDO-PostgreSQL',
@@ -690,26 +706,31 @@ class WebWizard extends Wizard implements SetupWizard
                 'To store users or preferences in a PostgreSQL database the PDO-PostgreSQL module for PHP is required.'
             )
         )));
-        $pgsqlSet->add(new ClassRequirement(array(
-            'optional'      => true,
-            'condition'     => 'Zend_Db_Adapter_Pdo_Pgsql',
-            'alias'         => mt('setup', 'Zend database adapter for PostgreSQL'),
+        $set->merge($dbSet);
+
+        $dbRequire = (new SetRequirement(array(
+            'optional'      => false,
+            'condition'     => $dbSet,
+            'title'         =>'Database',
+            'alias'         => 'PDO-MySQL OR PDO-PostgreSQL',
             'description'   => mt(
                 'setup',
-                'The Zend database adapter for PostgreSQL is required to access a PostgreSQL database.'
-            ),
-            'textAvailable' => mt(
+                'A database is mandatory, therefore at least one module '
+                . 'PDO-MySQL or PDO-PostgreSQL for PHP is required.'
+            )
+        )));
+
+        $set->add($dbRequire);
+
+        $set->add(new ConfigDirectoryRequirement(array(
+            'condition'     => Icinga::app()->getStorageDir(),
+            'title'         => mt('setup', 'Read- and writable storage directory'),
+            'description'   => mt(
                 'setup',
-                'The Zend database adapter for PostgreSQL is available.',
-                'setup.requirement.class'
-            ),
-            'textMissing'   => mt(
-                'setup',
-                'The Zend database adapter for PostgreSQL is missing.',
-                'setup.requirement.class'
+                'The Icinga Web 2 storage directory defaults to "/var/lib/icingaweb2", if' .
+                ' not explicitly set in the environment variable "ICINGAWEB_STORAGEDIR".'
             )
         )));
-        $set->merge($pgsqlSet);
 
         $set->add(new ConfigDirectoryRequirement(array(
             'condition'     => Icinga::app()->getConfigDir(),
